#include <galprop/DarkMatter.h>
#include "galpropy.hpp"

class PyDarkMatterProfile : DarkMatterProfile {
public:
    using DarkMatterProfile::DarkMatterProfile;
    double dist(double r_rnorm) const override {
        PYBIND11_OVERRIDE_PURE(double, DarkMatterProfile, dist, r_rnorm);
    }
};

class PyDarkMatter : DarkMatter {
public:
    using DarkMatter::DarkMatter;
    std::valarray<double> product_spectrum(
        IsotopeNumber ison, const std::valarray<double>& ekn) const override {
        PYBIND11_OVERRIDE_PURE(std::valarray<double>, DarkMatter,
                               product_spectrum, ison, ekn);
    }
    void configure(const config& cfg) override {
        PYBIND11_OVERRIDE(void, DarkMatter, configure, cfg);
    }
};

void bind_DarkMatter(py::module_ &m)
{
    // ------ dark matter profiles ------
    py::class_<DarkMatterProfile, PyDarkMatterProfile,
               std::shared_ptr<DarkMatterProfile>>(m, "DarkMatterProfile")
        .def(py::init<>())
        .def_readwrite("rho0", &DarkMatterProfile::rho0);

    py::class_<DMProfileGeneral, DarkMatterProfile,
               std::shared_ptr<DMProfileGeneral>>(m, "DMProfileGeneral")
        .def(py::init<double, double, double, double>())
        .def("dist", &DMProfileGeneral::dist)
        .def_readwrite("alpha", &DMProfileGeneral::alpha)
        .def_readwrite("beta", &DMProfileGeneral::beta)
        .def_readwrite("gamma", &DMProfileGeneral::gamma)
        .def_readwrite("R_rnorm", &DMProfileGeneral::R_rnorm);

    py::class_<DMProfileEinasto, DarkMatterProfile,
               std::shared_ptr<DMProfileEinasto>>(m, "DMProfileEinasto")
        .def(py::init<double, double>())
        .def("dist", &DMProfileEinasto::dist)
        .def_readwrite("alpha", &DMProfileEinasto::alpha)
        .def_readwrite("R_rnorm", &DMProfileEinasto::R_rnorm);

    py::class_<DMProfileBurkert, DarkMatterProfile,
               std::shared_ptr<DMProfileBurkert>>(m, "DMProfileBurkert")
        .def(py::init<double>())
        .def("dist", &DMProfileBurkert::dist)
        .def_readwrite("Rcore_rnorm", &DMProfileBurkert::Rcore_rnorm);

    // ------ dark matter ------
    py::class_<DarkMatter, PyDarkMatter, std::shared_ptr<DarkMatter>>(
        m, "DarkMatter")
        .def(py::init<>())
        .def_readwrite("profile", &DarkMatter::profile)
        .def_readwrite("mode", &DarkMatter::mode)
        .def_readwrite("mass", &DarkMatter::mass)
        .def_readwrite("sigmav", &DarkMatter::sigmav)
        .def_readwrite("lifetime", &DarkMatter::lifetime);

    py::enum_<BasicDarkMatter::mode_type>(m, "BasicDarkMatterModeType")
        .value("annih", BasicDarkMatter::mode_type::annih)
        .value("decay", BasicDarkMatter::mode_type::decay);

    py::class_<BasicDarkMatter, DarkMatter, std::shared_ptr<BasicDarkMatter>>(
        m, "BasicDarkMatter")
        .def(py::init<>())
        .def("load_dndx", &BasicDarkMatter::load_dndx)
        .def("load_dmsp", &BasicDarkMatter::load_dmsp)
        .def("product_spectrum", &BasicDarkMatter::product_spectrum)
        .def("configure", &BasicDarkMatter::configure)
        .def_readwrite("mediator_mass", &BasicDarkMatter::mediator_mass)
        .def_readwrite("dndx_data", &BasicDarkMatter::dndx_data);
}
