#include <galprop/EnergyLoss.h>
#include "galpropy.hpp"

class PyEnergyLoss : public EnergyLoss
{
public:
    using EnergyLoss::EnergyLoss;
    int loss(Particle& p, const Galprop& g) override {
        PYBIND11_OVERRIDE_PURE(int, EnergyLoss, loss, p, g);
    }
};

void bind_EnergyLoss(py::module_ &m)
{
    py::class_<ISM_Density>(m, "ISM_Density")
        .def(py::init<>())
        .def(py::init<const Galaxy&, size_t>())
        .def("set", py::overload_cast<const Galaxy&, size_t>(&ISM_Density::set))
        .def_readwrite("H2", &ISM_Density::H2)
        .def_readwrite("HI", &ISM_Density::HI)
        .def_readwrite("HII", &ISM_Density::HII)
        .def_readwrite("He_H_ratio", &ISM_Density::He_H_ratio)
        .def_readwrite("protium", &ISM_Density::protium)
        .def_readwrite("neutral_H", &ISM_Density::neutral_H)
        .def_readwrite("neutral_He", &ISM_Density::neutral_He);

    py::class_<EnergyLoss, PyEnergyLoss>(m, "EnergyLoss")
        .def(py::init<>());

    py::class_<ElectronEnergyLoss>(m, "ElectronEnergyLoss")
        .def(py::init<>())
        .def("loss", &ElectronEnergyLoss::loss)
        .def("ionization", &ElectronEnergyLoss::ionization)
        .def("coulomb", &ElectronEnergyLoss::coulomb)
        .def("bremsstrahlung", &ElectronEnergyLoss::bremsstrahlung)
        .def("synchrotron", &ElectronEnergyLoss::synchrotron)
        .def("compton", &ElectronEnergyLoss::compton);

    py::class_<NucleusEnergyLoss>(m, "NucleusEnergyLoss")
        .def(py::init<>())
        .def("loss", &NucleusEnergyLoss::loss)
        .def("ionization", &NucleusEnergyLoss::ionization)
        .def("coulomb", &NucleusEnergyLoss::coulomb);

    py::class_<EnergyLossFallback>(m, "EnergyLossFallback")
        .def(py::init<>())
        .def("loss", &EnergyLossFallback::loss)
        .def_readwrite("electron_energy_loss_fallback",
                       &EnergyLossFallback::electron_energy_loss_fallback)
        .def_readwrite("nucleus_energy_loss_fallback",
                       &EnergyLossFallback::nucleus_energy_loss_fallback);
}
