#include <galprop/GCR.h>
#include <galprop/DarkMatter.h>
#include "galpropy.hpp"


class PyPopulator : public GCR::Populator {
public:
    using GCR::Populator::Populator;

    using ptr_t = std::shared_ptr<Particle>;
    using map_t = std::map<std::string, ptr_t>;
    map_t populate(const Galprop& g) const override {
        PYBIND11_OVERRIDE_PURE(map_t, GCR::Populator, populate, g);
    }
};


void bind_GCR(py::module_ &m)
{
    using vecptrpop = std::vector<std::shared_ptr<GCR::Populator>>;
    py::bind_vector<vecptrpop>(m, "VecPtrPopulator");
    py::implicitly_convertible<py::list, vecptrpop>();
    py::implicitly_convertible<py::tuple, vecptrpop>();

    py::class_<GCR> gcr(m, "GCR");
    gcr.def(py::init<>())
        .def("populate", &GCR::populate)
        .def("__getitem__",
             [](const GCR& self, const std::string& name) {
                return self[name]; },
             "get by name")
        .def("__getitem__",
             [](const GCR &self, std::size_t i) {
                 if (i >= self.size()) throw py::index_error("out of range");
                 return self[i]; },
            "get by index")
        .def("__len__", [](const GCR& self) { return self.size(); })
        .def("__iter__", [](const GCR& self)
             { return py::make_iterator(self.begin(), self.end()); },
             py::keep_alive<0, 1>())
        .def("push_back", [](GCR& self, std::shared_ptr<Particle> p) {
             self.push_back(p); })
        .def_readwrite("populators", &GCR::populators,
                       py::return_value_policy::reference_internal);

    py::class_<GCR::Populator, PyPopulator, std::shared_ptr<GCR::Populator>>(
        gcr, "Populator")
        .def(py::init_alias<>())
        .def("populate", &GCR::Populator::populate);

    py::class_<PopPrimaryNucV54, GCR::Populator,
               std::shared_ptr<PopPrimaryNucV54>>(m, "PopPrimaryNucV54")
        .def(py::init<>());
    py::class_<PopPrimaryEleV54, GCR::Populator,
               std::shared_ptr<PopPrimaryEleV54>>(m, "PopPrimaryEleV54")
        .def(py::init<>());
    py::class_<PopSecondaryV54, GCR::Populator,
               std::shared_ptr<PopSecondaryV54>>(m, "PopSecondaryV54")
        .def(py::init<>());

    // dark matter TODO
    py::class_<PopPrimaryDMSP, GCR::Populator,
               std::shared_ptr<PopPrimaryDMSP>>(m, "PopPrimaryDMSP")
        .def(py::init<>()).def_readwrite("dm", &PopPrimaryDMSP::dm);
    py::class_<PopSecondaryAntinucleus, GCR::Populator,
               std::shared_ptr<PopSecondaryAntinucleus>>(
        m, "PopSecondaryAntinucleus").def(py::init<>());
}
