#include <galprop/PrimaryParticle.h>
#include "galpropy.hpp"

class PyPrimaryParticle : PrimaryParticle {
public:
    using PrimaryParticle::PrimaryParticle;
    bool depends_on(const Particle& p, Galprop& g) const override {
        PYBIND11_OVERRIDE(bool, PrimaryParticle, depends_on, p, g);
    }
    void fill_source_functions(Galprop& g) override {
        PYBIND11_OVERRIDE(void, PrimaryParticle, fill_source_functions, g);
    }
};

class PyPrimaryNucleus : PrimaryNucleus {
public:
    using PrimaryNucleus::PrimaryNucleus;
    bool depends_on(const Particle& p, Galprop& g) const override {
        PYBIND11_OVERRIDE(bool, PrimaryNucleus, depends_on, p, g);
    }
    void fill_source_functions(Galprop& g) override {
        PYBIND11_OVERRIDE(void, PrimaryNucleus, fill_source_functions, g);
    }
    double abundance(void) const override {
        PYBIND11_OVERRIDE(double, PrimaryNucleus, abundance,);
    }
};

void bind_PrimaryParticle(py::module_ &m)
{
    py::class_<PrimaryParticle, PyPrimaryParticle, Particle,
               std::shared_ptr<PrimaryParticle>>(m, "PrimaryParticle")
        .def(py::init<std::shared_ptr<const ParticleInfo>,
                      std::shared_ptr<const Galaxy>,
                      const std::string &>())
        .def_readwrite("spec_ptr", &PrimaryParticle::spec_ptr)
        .def_readwrite("dist_ptr", &PrimaryParticle::dist_ptr);

    py::class_<PrimaryNucleus, PyPrimaryNucleus, PrimaryParticle,
               std::shared_ptr<PrimaryNucleus>>(m, "PrimaryNucleus")
        .def(py::init<std::shared_ptr<const ParticleInfo>,
                      std::shared_ptr<const Galaxy>,
                      const std::string &>());
}
