#include <pybind11/stl.h>
#include <galprop/SolarModulation.h>

#include "galpropy.hpp"

class PySolarModulation : public SolarModulation {
public:
    using SolarModulation::SolarModulation;

    std::valarray<double> operator()(
        const std::valarray<double>& ekin_toa,
        const std::valarray<double>& ekin_is,
        const std::valarray<double>& flux_is,
        const ParticleNumber& ZA) const override {
        PYBIND11_OVERRIDE_PURE_NAME(std::valarray<double>, SolarModulation,
                                    "__call__", operator(),
                                    ekin_toa, ekin_is, flux_is, ZA);
    }
};

void bind_SolarModulation(py::module_ &m)
{
    py::class_<SolarModulation, PySolarModulation>(m, "SolarModulation")
        .def(py::init<>())
        .def("__call__", &SolarModulation::operator());

    py::class_<NoModulation, SolarModulation>(m, "NoModulation")
        .def(py::init<>());

    py::class_<ForceField, SolarModulation>(m, "ForceField")
        .def(py::init<double>())
        .def_readwrite("phi", &ForceField::phi);
}
