""" HelMod solar modulation """
import tempfile
import subprocess
import numpy as np

from ._bind_galpropcc import SolarModulation

class HelMod(SolarModulation):
    def __init__(self, particle, time_window):
        self.particle = particle
        self.time_window = time_window
        SolarModulation.__init__(self)

    def __call__(self, ekin_toa, ekin_is, flux_is, ZA):
        with tempfile.NamedTemporaryFile(delete=True) as tmpf:
            tmpname = tmpf.name
            np.savetxt(tmpname, np.vstack([ekin_is, flux_is]).T)
            subprocess.run(['helmod', '-i', tmpname, '-o', tmpname,
                            '-t', self.time_window, '-p', self.particle],
                           check=True)
            _, flux_toa = np.loadtxt(tmpname, unpack=True)
        return flux_toa
