""" CR sources """

from ._bind_galpropcc import (
    Galprop, GCR, SrcSpecSBPL, EleSrcDistParameterized, MapPtrParticle,
    PrimaryParticle)

class PopPrimaryElecSBPL(GCR.Populator):
    def __init__(self, *, segments=None, cutoff_rig=None):
        GCR.Populator.__init__(self)
        self._name = "primary_electrons"
        self._spectral = SrcSpecSBPL()
        self._spectral.abundance = 1
        self._spatial = EleSrcDistParameterized()
        if segments is not None:
            self.set_segments(segments)
        if cutoff_rig is not None:
            self.set_cutoff_rig(cutoff_rig)

    def set_segments(self, segments: list[tuple]) -> None:
        self._spectral.segments = segments

    def set_cutoff_rig(self, rig: float) -> None:
        self._spectral.cutoff_rig = rig

    def set_norm_ekn_GeV(self, v_GeV: float) -> None:
        self._spectral.set_norm_ekn_GeV(v_GeV)

    def set_norm_rig_GV(self, v_GV: float) -> None:
        self._spectral.set_norm_rig_GV(v_GV)

    def populate(self, g: Galprop) -> MapPtrParticle:
        if not g.galdef.primary_electrons:
            return MapPtrParticle()
        self._spatial.pG = g
        iele = g.particle_lib.get(-1, 0)
        pele = PrimaryParticle(iele, g.galaxy, self._name)
        pele.spec_ptr = self._spectral
        pele.dist_ptr = self._spatial
        particle_map = MapPtrParticle()
        particle_map[self._name] = pele
        return particle_map
